package lib {

import flash.desktop.Clipboard;
import flash.desktop.ClipboardFormats;
import flash.desktop.NativeApplication;
import flash.desktop.SystemTrayIcon;
import flash.display.BitmapData;
import flash.display.NativeMenu;
import flash.display.NativeMenuItem;
import flash.events.Event;
import flash.events.MouseEvent;
import flash.events.NativeDragEvent;
import flash.filesystem.File;

import mx.core.BitmapAsset;
import mx.core.IMXMLObject;
import mx.core.UIComponent;
import mx.events.FlexEvent;
import mx.managers.DragManager;

public class MainLogic implements IMXMLObject {
	
	/** 再生中のFLVファイル */
	private var flvFile:File = new File();
	
	/** システムトレイに表示するアイコンBitmap */
	[Embed(source="assets/icons/trayIcon.png")]
	private var icon16:Class;
	
	/** システムトレイアイコン */
	private var systemTrayIcon:SystemTrayIcon;
	
	/** システムトレイアイコンメニュー */
	private var systemTrayMenu:NativeMenu;
	
	/** システムトレイに表示するアイコン */
	private var iconBitmap:BitmapData;
	
	
	/**
	 * コンストラクタ
	 */
	public function MainLogic() {
		super();
	}


	/**
	 * initialized
	 * @param document
	 * @param id
	 */	
	public function initialized(document:Object, id:String):void {
		_document = document as UIComponent;
		_id = id;
		
	}
	
	
	/**
	 * onCreationCompleteHandler
	 * @param event
	 */	
	public function onCreationCompleteHandler(event:FlexEvent):void {
		
		//addEventListener
		view.addEventListener(NativeDragEvent.NATIVE_DRAG_ENTER, onNativeDragEnterHandler);
		view.addEventListener(NativeDragEvent.NATIVE_DRAG_DROP, onNativeDragDropHandler);
		
		//システムトレイアイコンをサポートしているか
		if (NativeApplication.supportsSystemTrayIcon) {
			systemTrayIcon = NativeApplication.nativeApplication.icon as SystemTrayIcon;
			//アイコン表示
			setSystemTrayIcon();
		}
		
	}
	
	
	/**
	 * DragEnter時の処理
	 * @param event
	 */
	private function onNativeDragEnterHandler(event:NativeDragEvent):void {
		trace(">>onNativeDragEnterHandler", event.toString());
		
		//ドラッグされてきているデータを取得
		var transferable:Clipboard = event.clipboard;
		
		//FLVファイルの場合はドロップを許可する
		if(transferable.hasFormat(ClipboardFormats.FILE_LIST_FORMAT)){
			var dropfiles:Array = transferable.getData(ClipboardFormats.FILE_LIST_FORMAT) as Array;
			for each (var file:File in dropfiles){
				if(file.extension == "flv"){
					DragManager.acceptDragDrop(view);
					break;
				}
			}
		}
		
	}
	
	
	/**
	 * ファイルをドロップした際の処理
	 * @param event
	 */
	private function onNativeDragDropHandler(event:NativeDragEvent):void {
		trace(">>onNativeDragDropHandler", event.toString());
		
		//ドラッグされてきているデータを取得
		var transferable:Clipboard = event.clipboard;
		
		var dropfiles:Array = transferable.getData(ClipboardFormats.FILE_LIST_FORMAT) as Array;
		for each (var file:File in dropfiles){
			if(file.extension == "flv"){
				setFlv(file);
				break;
			}
		}
		
	}
	
	
	/**
	 * 取得したFLVをVideoDisplayで再生させる
	 * @param file
	 */
	private function setFlv(file:File):void {
		flvFile = file;
		
		//再生
		view.videoDisplay.source = flvFile.nativePath;
		
		//再生しているファイル名の表示
		view.flvNameLabel.text = flvFile.name;
	}
	
	
	/** ボリュームコントロール */
	public function volumeUpBtnClickHandler(event:MouseEvent):void {
		if (view.videoDisplay.volume < 1) {
			var volume:int = view.videoDisplay.volume*100;
			volume += 5;
			view.videoDisplay.volume = volume/100;
		}
	}
	/** ボリュームコントロール */
	public function volumeDownBtnClickHandler(event:MouseEvent):void {
		if (view.videoDisplay.volume > 0) {
			var volume:int = view.videoDisplay.volume*100;
			volume -= 5;
			view.videoDisplay.volume = volume/100;
		}
	}
	
	
	/** 
	 * システムトレイアイコン設定
	 */
	private function setSystemTrayIcon():void {
		systemTrayMenu = new NativeMenu();
		systemTrayIcon.menu = systemTrayMenu;
		
		//アイコンの設定
		iconBitmap = (new icon16() as BitmapAsset).bitmapData;
		systemTrayIcon.bitmaps = [iconBitmap];
		
		//ロールオーバーした際のtooltip表示
		systemTrayIcon.tooltip = "FLVPlayerExample";
		
		//メニュー設定
		setMenuItems();
	}
	
	
	/**
	 * システムトレイアイコンのメニューを初期化
	 */ 
	private function setMenuItems():void{
		//再生
		var playMenuItem:NativeMenuItem = new NativeMenuItem("再生");
		playMenuItem.addEventListener(Event.SELECT, playMenuItemSelectHandler);
		//一次停止
		var pauseMenu:NativeMenuItem = new NativeMenuItem("一時停止");
		pauseMenu.addEventListener(Event.SELECT, pauseMenuSelectHandler);
		//停止
		var stopMenu:NativeMenuItem = new NativeMenuItem("停止");
		stopMenu.addEventListener(Event.SELECT, stopMenuSelectHandler);
		//終了
		var exitMenuItem:NativeMenuItem = new NativeMenuItem("終了");
		exitMenuItem.addEventListener(Event.SELECT, exitMenuItemSelectHandler);
		
		//メニューの追加
		systemTrayMenu.addItem(playMenuItem);
		systemTrayMenu.addItem(pauseMenu);
		systemTrayMenu.addItem(stopMenu);
		systemTrayMenu.addItem(exitMenuItem);
	}
	
	
	/** 
	 * systemTrayMenu「終了」の際の処理
	 */
	private function exitMenuItemSelectHandler(event:Event):void{
		//アプリケーションの終了
		view.exit();
	}
	/** 
	 * systemTrayMenu「停止」の際の処理
	 */
	private function stopMenuSelectHandler(event:Event):void{
		//アプリケーションの最小化
		view.videoDisplay.stop();
	}
	/** 
	 * systemTrayMenu「一時停止」の際の処理
	 */
	private function pauseMenuSelectHandler(event:Event):void{
		//一時停止
		view.videoDisplay.pause();
	}
	/** 
	 * systemTrayMenu「再生」の際の処理
	 */
	 
	private function playMenuItemSelectHandler(event:Event):void{
		//再生
		view.videoDisplay.play();
	}
	
	
	/**
	 * システムトレイアイコンの削除
	 */
	public function removeSystemTrayIcon():void{
		//trace("システムトレイアイコン削除");
		systemTrayIcon.bitmaps = new Array();
	}
	
	
	//________________________________________________________________________________
	//                                                                    id document
	/** MXMLドキュメント */
	private var _document:UIComponent;
	
	/** ドキュメント識別子 */
	private var _id:String;
	
	
	/**
	 * document
	 * @return 
	 */	
	public final function get document():UIComponent {
		return _document;
	}
	
	
	/**
	 * id
	 * @return 
	 */	
	public final function get id():String {
		return _id;
	}
	
	
	//________________________________________________________________________________
	//                                                                       view取得
	/** 画面 */
	private var _view:FLVPlayerExample;
	
	/**
	 * 画面取得
	 */		
	public function get view():FLVPlayerExample {
		if (_view == null) {
			_view = document as FLVPlayerExample;
		}
		return _view;
	}
	
	/**
	 * 画面セット
	 */
	public function set view(view:FLVPlayerExample):void {
		_view = view;
	}
	
}
}